.\" Automatically generated by Pandoc 2.11.4
.\"
.TH "regex_sub" "3" "April 5, 2021" "rationL 0.1.0" "rationL User Manual"
.hy
.SH NAME
.PP
regex_sub \[en] Replaces matches of a regular expression in a string
.SH SYNOPSIS
.IP
.nf
\f[C]
#include <rationl.h>

char* regex_sub(reg_t re, char* input, char* replace);
\f[R]
.fi
.SH DESCRIPTION
.PP
\f[B]regex_sub()\f[R] searches maximal substring that the regular
expression given as parameter matches in \f[I]intput\f[R] and replaces
it with the \f[I]replace\f[R] value.
Returns the result of the remplacement.
The return value will need to be freed by the programmer.
If no occurence is found the string returned is simply the same and the
\f[I]input\f[R] string.
.PP
\f[I]re\f[R] The regular expressions to match against
.PP
\f[I]input\f[R] The input string to replace from.
.PP
\f[I]replace\f[R] The string to replace occurences found with.
.SH RETURN VALUE
.PP
\f[B]regex_sub()\f[R] The input string with the replaced values.
If there is no match in the input string, the return value is the same
as the input.
The return value will need to be freed by the programmer.
.SH EXAMPLES
.PP
\f[B]Program that replaces any version number in a file with the latest
one\f[R]
.IP
.nf
\f[C]
#include <err.h>
#include <rationl.h>
#include <stdlib.h>
#include <stdio.h>

int main(int argc, char **argv)
{
    if (argc != 2)
        errx(1, \[dq]No input file found.\[dq]);

    reg_t re = regex_compile(\[dq]v(\[rs]\[rs]d+)\[rs]\[rs].(\[rs]\[rs]d+)\[rs]\[rs].(\[rs]\[rs]d+)\[dq]);

    char *line_buf = NULL;
    size_t line_buf_size = 0;
    int line_count = 0;
    ssize_t line_size;
    FILE *fp = fopen(argv[1], \[dq]r\[dq]);
    if (!fp)
    {
        errx(EXIT_FAILURE, \[dq]Error opening file \[aq]%s\[aq]\[rs]n\[dq], argv[1]);
    }

    char **groups;

    while ((line_size = getline(&line_buf, &line_buf_size, fp) >= 0))
    {
        char* replaced_value = regex_sub(re, line_buf, \[dq]v2.5.0\[dq]);
        printf(\[dq]old:%snew:%s\[dq], line_buf, replaced_value);
        free(replaced_value);
    }

    free(line_buf);
    fclose(fp);
    regex_free(re);
    return 0;
}
\f[R]
.fi
.SH SEE ALSO
.IP
.nf
\f[C]
regex_match(3) regex_search(3)
\f[R]
.fi
